!===============================================================================
! Program : psum1.f90
! Purpose : Sums up elemnts of an array, a(n), using block distribution
!           Illustrates shrinking of arrays
!
! Compile : mpif90 -o psum1.x psum1.f90 -O2
!===============================================================================
program main
  implicit none
  include 'mpif.h'
  integer(4) :: i
  integer(4) :: ista
  integer(4) :: iend
  integer(4) :: sum
  integer(4) :: ssum
  integer(4) :: ierr
  integer(4) :: iproc
  integer(4) :: nproc
  integer(4), parameter :: n = 1000
  integer(4), allocatable :: a(:)
  call MPI_INIT(ierr)
  call MPI_COMM_SIZE(MPI_COMM_WORLD, nproc, ierr)
  call MPI_COMM_RANK(MPI_COMM_WORLD, iproc, ierr)
  call para_range(1, n, nproc, iproc, ista, iend)
  if ( .not. allocated(a) ) allocate( a(ista:iend) )
  do i = ista, iend
     a(i) = i
  end do
  sum = 0.0
  do i = ista, iend
     sum = sum + a(i)
  end do
  call MPI_REDUCE(sum, ssum, 1, MPI_INTEGER,MPI_SUM, 0,MPI_COMM_WORLD, ierr)
  sum = ssum
  if ( iproc == 0 ) write(6,*)'sum =', sum
  if ( allocated(a) ) deallocate(a)
  call MPI_FINALIZE(ierr)
end program main

!===============================================================================
! Calculates iteration range and/or array dimension for each MPI process
! Adapted from "RS/6000 SP: Practical MPI Programming", IBM red book
!===============================================================================
subroutine para_range(n1, n2, nprocs, irank, ista, iend)
  integer(4) :: n1        ! Lowest value of iteration variable
  integer(4) :: n2        ! Highest value of iteration variable
  integer(4) :: nprocs    ! # cores
  integer(4) :: irank     ! Iproc (rank)
  integer(4) :: ista      ! Start of iterations for rank iproc
  integer(4) :: iend      ! End of iterations for rank iproc
  iwork1 = ( n2 - n1 + 1 )  /  nprocs
  iwork2 = MOD(n2 - n1 + 1, nprocs)
  ista = irank * iwork1 + n1 + MIN(irank, iwork2)
  iend = ista + iwork1 - 1
  if ( iwork2 > irank ) iend = iend + 1
  return
end subroutine para_range
